from typing import List, Optional, Tuple, Type

from langchain.base_language import BaseLanguageModel
from langchain.chains import RetrievalQA
from langchain.chains.base import Chain
from langchain.chains.retrieval_qa.base import BaseRetrievalQA
from langchain.chat_models import ChatOpenAI
from langchain.embeddings.base import Embeddings
from langchain.schema import Document

from chat.ask_codebase.store.qdrant import QdrantWrapper as Q
from chat.ask_codebase import DEBUG


class SimpleQA:
    # 3.5 cannot map code as expect, maybe we should change prompt..
    llm: BaseLanguageModel = ChatOpenAI(temperature=0.1, model_name="gpt-4")  # type: ignore
    reduce_llm: BaseLanguageModel = ChatOpenAI(  # type: ignore
        temperature=0.1, model_name="gpt-4"
    )

    def __init__(
        self,
        # source_name: str,
        # embedding_cls: Type[Embeddings],
        q: Q,
    ) -> None:
        # self._source_name = source_name
        # self._embedding_cls = embedding_cls
        self._q = q

        self._chain: Optional[Chain] = None

    @property
    def chain(self) -> Chain:
        if self._chain is None:
            # q = Q.reuse(self._source_name, self._embedding_cls)
            retriver = self._q.qdrant.as_retriever(
                search_type="similarity", search_kwargs={"k": 20}
            )
            chain: BaseRetrievalQA = RetrievalQA.from_chain_type(
                llm=self.llm,
                chain_type_kwargs={
                    "reduce_llm": self.reduce_llm,
                },
                chain_type="map_reduce",
                retriever=retriver,
                return_source_documents=True,
                verbose=DEBUG,
            )
            self._chain = chain

        return self._chain

    def run(self, question: str) -> Tuple[str, List[Document]]:
        answer = self.chain(question)

        return answer["result"], answer["source_documents"]
