from typing import Iterable, Iterator, List, Tuple

from langchain.embeddings.base import Embeddings
from langchain.schema import Document
from loguru import logger

Vector = List[float]


class EmbeddingWrapper:
    # class Batch:
    #     def __init__(self, embedding: Embeddings, batch_count: int):
    #         self._batch_count = batch_count
    #         self._current_count = 0
    #         self._docs: List[Document] = []
    #         self._embedding = embedding

    #     def embed(self, doc: Document):
    #         self._docs.append(doc)

    #         if len(self._docs) >= self._batch_count:
    #             self._flush()

    #     def _flush(self):
    #         self._embedding.embed_documents(self._docs)
    #         self._docs.clear()

    def __init__(self, embedding: Embeddings) -> None:
        self._embedding = embedding

    def embed(
        self, documents: Iterable[Document], batch_count: int = 100
    ) -> Iterator[Tuple[Vector, Document]]:
        """
        Embed documents in a lazy way
        """
        buffer: List[Document] = []
        for doc in documents:
            buffer.append(doc)
            if len(buffer) >= batch_count:
                # yield self.embed_many(buffer)
                vectors = self.embed_many(buffer)
                yield from zip(vectors, buffer)
                buffer.clear()

        if buffer:
            vectors = self.embed_many(buffer)
            yield from zip(vectors, buffer)
            buffer.clear()

    def embed_many(self, documents: List[Document]) -> List[Vector]:
        logger.info(f"Start embedding {len(documents)} documents")

        texts = [doc.page_content for doc in documents]
        embeddings = self._embedding.embed_documents(texts)

        logger.info("Finish Embedding.")
        return embeddings
