import json
from typing import Iterable, Iterator, Tuple

from langchain.schema import Document

from chat.ask_codebase.indexing.embedding import Vector


def dump_file(filepath: str, data: Iterable[Tuple[Vector, Document]]):
    """
    Dump vectors to a file
    each line is a json object with the following fields:
        vector, metadata, page_content
    """
    with open(filepath, "w") as f:
        for v, d in data:
            json.dump(
                {
                    "vector": v,
                    "metadata": d.metadata,
                    "page_content": d.page_content,
                },
                f,
            )
            f.write("\n")  # make each line a json object


def load_file(filepath: str) -> Iterator[Tuple[Vector, Document]]:
    """
    Load vectors from a file
    each line is a json object with the following fields:
        vector, metadata, page_content
    """
    with open(filepath, "r") as f:
        for line in f:
            data = json.loads(line)

            metadata = data.get("metadata")
            page_content = data.get("page_content")
            doc = Document(metadata=metadata, page_content=page_content)
            vector = data.get("vector")
            yield (vector, doc)
