from datetime import datetime
from functools import wraps


def func_timer(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        t0 = datetime.now()
        res = func(*args, **kwargs)
        t1 = datetime.now()

        duration = t1 - t0
        print(
            f"[{func.__name__}] Start: {t0.strftime('%Y-%m-%d %H:%M:%S')}, "
            f"End: {t1.strftime('%Y-%m-%d %H:%M:%S')}, "
            f"Duration: {duration}."
        )
        return res

    return wrapper


def func_log(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        print(f"[{func.__name__}] Start")
        res = func(*args, **kwargs)
        print(f"[{func.__name__}] End")
        return res

    return wrapper
